@@ -133,7 +133,9 @@ class PurchaseTransactionDTO(BaseTransactionDTO):
                  recurring: bool, interchange: Optional[int], ecommerce: bool, card_present: bool,
                  payment_method: Optional[str], digital_wallet: Optional[str], card_verification_data,
                  card_network: Optional[str], tags: Optional[Dict[str, str]],
-                 relationships: Optional[Dict[str, Relationship]]):
+                 relationships: Optional[Dict[str, Relationship]], gross_interchange: Optional[str],
+                 cash_withdrawal_amount: Optional[int], currency_conversion: Optional[CurrencyConversion],
+                 rich_merchant_data: Optional[RichMerchantData]):
         BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
         self.type = 'purchaseTransaction'
         self.attributes["cardLast4Digits"] = card_last_4_digits
@@ -147,6 +149,10 @@ class PurchaseTransactionDTO(BaseTransactionDTO):
         self.attributes["digitalWallet"] = digital_wallet
         self.attributes["cardVerificationData"] = card_verification_data
         self.attributes["cardNetwork"] = card_network
+        self.attributes["grossInterchange"] = gross_interchange
+        self.attributes["cashWithdrawalAmount"] = cash_withdrawal_amount
+        self.attributes["currencyConversion"] = currency_conversion
+        self.attributes["richMerchantData"] = rich_merchant_data
 
     @staticmethod
     def from_json_api(_id, _type, attributes, relationships):
@@ -157,14 +163,17 @@ class PurchaseTransactionDTO(BaseTransactionDTO):
             attributes["recurring"], attributes.get("interchange"), attributes.get("ecommerce"),
             attributes.get("cardPresent"), attributes.get("paymentMethod"), attributes.get("digitalWallet"),
             attributes.get("cardVerificationData"), attributes.get("cardNetwork"), attributes.get("tags"),
-            relationships)
+            relationships, attributes.get("grossInterchange"), attributes.get("cashWithdrawalAmount"),
+            CurrencyConversion.from_json_api(attributes.get("currencyConversion")),
+            RichMerchantData.from_json_api(attributes.get("richMerchantData")))
 
 
 class AtmTransactionDTO(BaseTransactionDTO):
     def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int,
                  summary: str, card_last_4_digits: str, atm_name: str, atm_location: Optional[str], surcharge: int,
                  interchange: Optional[int], card_network: Optional[str],
-                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
+                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]],
+                 gross_interchange: Optional[str], currency_conversion: Optional[CurrencyConversion]):
         BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
         self.type = 'atmTransaction'
         self.attributes["cardLast4Digits"] = card_last_4_digits
@@ -173,6 +182,8 @@ class AtmTransactionDTO(BaseTransactionDTO):
         self.attributes["surcharge"] = surcharge
         self.attributes["interchange"] = interchange
         self.attributes["cardNetwork"] = card_network
+        self.attributes["grossInterchange"] = gross_interchange
+        self.attributes["currencyConversion"] = currency_conversion
 
     @staticmethod
     def from_json_api(_id, _type, attributes, relationships):
@@ -180,7 +191,8 @@ class AtmTransactionDTO(BaseTransactionDTO):
                                  attributes["amount"], attributes["balance"], attributes["summary"],
                                  attributes["cardLast4Digits"], attributes["atmName"], attributes.get("atmLocation"),
                                  attributes["surcharge"], attributes.get("interchange"), attributes.get("cardNetwork"),
-                                 attributes.get("tags"), relationships)
+                                 attributes.get("tags"), relationships, attributes.get("grossInterchange"),
+                                 CurrencyConversion.from_json_api(attributes.get("currencyConversion")))
 
 
 class FeeTransactionDTO(BaseTransactionDTO):
@@ -201,7 +213,8 @@ class CardTransactionDTO(BaseTransactionDTO):
                  summary: str, card_last_4_digits: str, merchant: Optional[Merchant], recurring: Optional[bool],
                  interchange: Optional[int], payment_method: Optional[str], digital_wallet: Optional[str],
                  card_verification_data: Optional[Dict], card_network: Optional[str], tags: Optional[Dict[str, str]],
-                 relationships: Optional[Dict[str, Relationship]]):
+                 relationships: Optional[Dict[str, Relationship]], gross_interchange: Optional[str],
+                 currency_conversion: Optional[CurrencyConversion], rich_merchant_data: Optional[RichMerchantData]):
         BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
         self.type = 'cardTransaction'
         self.attributes["cardLast4Digits"] = card_last_4_digits
@@ -212,7 +225,9 @@ class CardTransactionDTO(BaseTransactionDTO):
         self.attributes["digitalWallet"] = digital_wallet
         self.attributes["cardVerificationData"] = card_verification_data
         self.attributes["cardNetwork"] = card_network
-
+        self.attributes["grossInterchange"] = gross_interchange
+        self.attributes["currencyConversion"] = currency_conversion
+        self.attributes["richMerchantData"] = rich_merchant_data
 
     @staticmethod
     def from_json_api(_id, _type, attributes, relationships):
@@ -222,7 +237,10 @@ class CardTransactionDTO(BaseTransactionDTO):
                                   attributes.get("recurring"), attributes.get("interchange"),
                                   attributes.get("paymentMethod"), attributes.get("digitalWallet"),
                                   attributes.get("cardVerificationData"), attributes.get("cardNetwork"),
-                                  attributes.get("tags"), relationships)
+                                  attributes.get("tags"), relationships, attributes.get("grossInterchange"),
+                                  CurrencyConversion.from_json_api(attributes.get("currencyConversion")),
+                                  RichMerchantData.from_json_api(attributes.get("richMerchantData"))
+                                  )
 
 
 class CardReversalTransactionDTO(BaseTransactionDTO):
@@ -378,7 +396,7 @@ class PaymentAdvanceTransactionDTO(BaseTransactionDTO):
 
 class RepaidPaymentAdvanceTransactionDTO(BaseTransactionDTO):
     def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int, summary: str,
-                 reason: str, tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
+                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
         BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
         self.type = 'repaidPaymentAdvanceTransaction'
 
@@ -389,6 +407,7 @@ class RepaidPaymentAdvanceTransactionDTO(BaseTransactionDTO):
                                                   attributes["amount"], attributes["balance"], attributes["summary"],
                                                   attributes.get("tags"), relationships)
 
+
 class RewardTransactionDTO(BaseTransactionDTO):
     def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int, summary: str,
                  receiver_counterparty: Counterparty, tags: Optional[Dict[str, str]],
@@ -417,6 +436,7 @@ class PaymentCanceledTransactionDTO(BaseTransactionDTO):
                                              attributes["direction"], attributes["amount"], attributes["balance"],
                                              attributes["summary"], attributes.get("tags"), relationships)
 
+
 class ChargebackTransactionDTO(BaseTransactionDTO):
     def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int, summary: str,
                  counterparty: Counterparty, tags: Optional[Dict[str, str]],
@@ -433,6 +453,79 @@ class ChargebackTransactionDTO(BaseTransactionDTO):
                                         Counterparty.from_json_api(attributes.get("counterparty")),
                                         attributes.get("tags"), relationships)
 
+
+
+class AccountLowBalanceClosureTransactionDTO(BaseTransactionDTO):
+    def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int, summary: str,
+                 receiver_counterparty: Counterparty, tags: Optional[Dict[str, str]],
+                 relationships: Optional[Dict[str, Relationship]]):
+        BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
+        self.type = 'accountLowBalanceClosureTransaction'
+        self.attributes["receiverCounterparty"] = receiver_counterparty
+
+    @staticmethod
+    def from_json_api(_id, _type, attributes, relationships):
+        return AccountLowBalanceClosureTransactionDTO(_id, date_utils.to_datetime(attributes["createdAt"]),
+                                                   attributes["direction"], attributes["amount"], attributes["balance"],
+                                                   attributes["summary"],
+                                                   Counterparty.from_json_api(attributes.get("receiverCounterparty")),
+                                                   attributes.get("tags"), relationships)
+
+
+class NegativeBalanceCoverageTransactionDTO(BaseTransactionDTO):
+    def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int, summary: str,
+                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
+        BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
+        self.type = 'negativeBalanceCoverageTransaction'
+
+    @staticmethod
+    def from_json_api(_id, _type, attributes, relationships):
+        return NegativeBalanceCoverageTransactionDTO(_id, date_utils.to_datetime(attributes["createdAt"]),
+                                                  attributes["direction"], attributes["amount"], attributes["balance"],
+                                                  attributes["summary"], attributes.get("tags"), relationships)
+
+
+class PushToCardTransactionDTO(BaseTransactionDTO):
+    def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int, summary: str,
+                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
+        BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
+        self.type = 'pushToCardTransaction'
+
+    @staticmethod
+    def from_json_api(_id, _type, attributes, relationships):
+        return PushToCardTransactionDTO(_id, date_utils.to_datetime(attributes["createdAt"]),
+                                        attributes["direction"], attributes["amount"], attributes["balance"],
+                                        attributes["summary"], attributes.get("tags"), relationships)
+
+
+class CheckPaymentTransactionDTO(BaseTransactionDTO):
+    def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int, summary: str,
+                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
+        BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
+        self.type = 'checkPaymentTransaction'
+
+    @staticmethod
+    def from_json_api(_id, _type, attributes, relationships):
+        return CheckPaymentTransactionDTO(_id, date_utils.to_datetime(attributes["createdAt"]),
+                                          attributes["direction"], attributes["amount"], attributes["balance"],
+                                          attributes["summary"], attributes.get("tags"), relationships)
+
+
+class ReturnedCheckPaymentTransactionDTO(BaseTransactionDTO):
+    def __init__(self, id: str, created_at: datetime, direction: str, amount: int, balance: int, summary: str,
+                 reason: str, tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
+        BaseTransactionDTO.__init__(self, id, created_at, direction, amount, balance, summary, tags, relationships)
+        self.attributes["reason"] = reason
+        self.type = 'returnedCheckPaymentTransaction'
+
+    @staticmethod
+    def from_json_api(_id, _type, attributes, relationships):
+        return ReturnedCheckPaymentTransactionDTO(_id, date_utils.to_datetime(attributes["createdAt"]),
+                                                  attributes["direction"], attributes["amount"], attributes["balance"],
+                                                  attributes["summary"], attributes["reason"], attributes.get("tags"),
+                                                  relationships)
+
+
 TransactionDTO = Union[OriginatedAchTransactionDTO, ReceivedAchTransactionDTO, ReturnedAchTransactionDTO,
                        ReturnedReceivedAchTransactionDTO, DishonoredAchTransactionDTO, BookTransactionDTO,
                        PurchaseTransactionDTO, AtmTransactionDTO, FeeTransactionDTO, CardTransactionDTO,
@@ -440,7 +533,18 @@ TransactionDTO = Union[OriginatedAchTransactionDTO, ReceivedAchTransactionDTO, R
                        InterestTransactionDTO, DisputeTransactionDTO, CheckDepositTransactionDTO,
                        ReturnedCheckDepositTransactionDTO, PaymentAdvanceTransactionDTO,
                        RepaidPaymentAdvanceTransactionDTO, RewardTransactionDTO, PaymentCanceledTransactionDTO,
-                       ChargebackTransactionDTO]
+                       ChargebackTransactionDTO, AccountLowBalanceClosureTransactionDTO,
+                       NegativeBalanceCoverageTransactionDTO, PushToCardTransactionDTO, CheckPaymentTransactionDTO,
+                       ReturnedCheckPaymentTransactionDTO]
+
+
+def transactions_mapper(_id, _type, attributes, relationships):
+    c = globals()
+    c_name = _type[0].upper() + _type[1:] + "DTO"
+    if c_name in c.keys():
+        return c[c_name].from_json_api(_id, _type, attributes, relationships)
+    else:
+        return RawUnitObject(_id, _type, attributes, relationships)
 
 
 class PatchTransactionRequest(BaseTransactionDTO, UnitRequest):
@@ -509,4 +613,3 @@ class ListTransactionParams(UnitParams):
         if self.include:
             parameters["include"] = self.include
         return parameters
-
